/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file Factory.h
 *
 *  \date Dec 20, 2010
 *  \author Alexandre Tuleu
 */

#ifndef BIOROB_CPP_BASE_FACTORY_H_
#define BIOROB_CPP_BASE_FACTORY_H_

#include <string>
#include <map>
#include <memory>

#include <stdexcept>

template <typename Key,typename Object>
class Factory{
public :
	static bool RegisterObjectES(const Key & key,
	                             const Object & object);

	static void RegisterObject(const Key & key,
	                           const Object & object);

	static bool IsRegistered(const Key & key);

	static bool UnregisterES(const Key & key);

	static void Unregister(const Key & key);

	static std::auto_ptr<Object> Create(const Key & key);

	static bool CreateES(const Key & key, std::auto_ptr<Object> & created);

private :
	typedef std::map<Key,Object*> Map;

	static void ThrowOutOfScope(const Key & key);


	static Map s_registered;
};

#include "Factory.ih"

#endif // LIBAMARSI_QUAD_FACTORY_H_
