/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file FactoryUTest.cpp
 *
 *  \date Dec 20, 2010
 *  \author Alexandre Tuleu
 */

#include "Factory.h"

#include "FactoryUTest.h"



MyObject::MyObject(const std::string & value)
	: d_value(value){
}

MyObject::~MyObject(){}

const std::string & MyObject::Name() const{
	return d_value;
}


void FactoryUTest::SetUp(){
	MyObject a("Tata"),e("Tete"),i("Titi"),o("Toto"),u("Tutu");
	Factory<std::string,MyObject>::RegisterObjectES(a.Name(),a);
	Factory<std::string,MyObject>::RegisterObjectES(e.Name(),e);
	Factory<std::string,MyObject>::RegisterObjectES(i.Name(),i);
}

void FactoryUTest::checkIfExistsOrNot(const std::string & name,bool exists){
	bool isRegistered(Factory<std::string,MyObject>::IsRegistered(name));
	EXPECT_EQ(exists,isRegistered);
}

void FactoryUTest::checkIfValueIsOk(const std::string & name){
	std::auto_ptr<MyObject> obj= Factory<std::string,MyObject>::Create(name);
	EXPECT_EQ(name,obj->Name());
}


void FactoryUTest::registerMyObject(MyObject & a){
	Factory<std::string,MyObject>::RegisterObject(a.Name(),a);
}

TEST_F(FactoryUTest,RegisterCorrectly){
	checkIfExistsOrNot("Tata",true);
	checkIfExistsOrNot("Tete",true);
	checkIfExistsOrNot("Titi",true);
	checkIfExistsOrNot("Toto",false);
	checkIfExistsOrNot("abcdefg",false);

	checkIfValueIsOk("Tata");
	checkIfValueIsOk("Tete");
	checkIfValueIsOk("Titi");


}

TEST_F(FactoryUTest,NoTwiceRegistering){
	MyObject a("Tata");
	try{
		registerMyObject(a);
		FAIL();
	}catch(std::logic_error & e){
		SUCCEED();
	}
}

TEST_F(FactoryUTest,Removal){
	try{
		Factory<std::string,MyObject>::Unregister("Blah");
		FAIL();
	}catch(std::logic_error & exception){
	}

	Factory<std::string,MyObject>::Unregister("Tata");
	checkIfExistsOrNot("Tata",false);
}
