/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * Copyright (c) 2012  Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file ParsersUTest.cpp
 *
 *  \date Jul 26, 2012
 *  \author Alexandre Tuleu
 */

#include "ParsersUTest.h"
#include <sstream>
#include "Parsers.h"


ParsersUTest::ParsersUTest() : d_root() {}
ParsersUTest::~ParsersUTest() {}

void ParsersUTest::SetUp(){
	d_root = options::Section::Ptr(new options::Section("",""));
	std::ostringstream os;
	os << "## This is the foo section" << std::endl
	   << "[foo]" << std::endl
	   << "# ignored" << std::endl
	   << "## a given option" << std::endl
	   << "# still ignored" << std::endl
	   << "a = foo # some comment" << std::endl
	   << "## a commented option" << std::endl
	   << "# b = bar" << std::endl
	   << "## a last option" << std::endl
	   << "c = baz" << std::endl
	   << "## should be ignored" << std::endl
	   << "# like this one" << std::endl
	   << "## but not this one" << std::endl
	   << "[foo.bar]" << std::endl
	   << "[foo.baz]" << std::endl
	   << " dd = dd" << std::endl
	   << std::endl;
	std::string f;
	f = os.str();
	std::istringstream iss(f);

	options::ConfigFileParser p;
	p.Parse(iss,d_root);
}

TEST_F(ParsersUTest,ParsesSectionsCorrectly){
	EXPECT_EQ(1,d_root->SubSections().size());
	ASSERT_NE(0,d_root->SubSections().size());
	options::Section::Ptr s = d_root->SubSections().begin()->second;
	EXPECT_STREQ("foo",s->Name().c_str());
	EXPECT_STREQ("This is the foo section",s->Comment().c_str());
	ASSERT_EQ(2,s->SubSections().size());
	EXPECT_STREQ("but not this one",s->SubSections().begin()->second->Comment().c_str());
}


TEST_F(ParsersUTest, ParseOptionsCorrectly){
	options::Section::Ptr s = d_root->SubSection("foo");

	AssertValid(s);
	EXPECT_EQ(2,s->Options().size());

	options::OptionBase::Ptr a = s->GetOption("a");
	options::OptionBase::Ptr c = s->GetOption("c");
	AssertValid(a);AssertValid(c);

	EXPECT_STREQ("foo",a->ValueAsString().c_str());
	EXPECT_STREQ("baz",c->ValueAsString().c_str());

	EXPECT_STREQ("a given option",a->Comment().c_str());
	EXPECT_STREQ("a last option",c->Comment().c_str());

}

TEST_F(ParsersUTest, PrintsSectionCorrectly){
	std::ostringstream os;
	os << "## This is the foo section" << std::endl
	   << "[foo]" << std::endl
	   << std::endl
	   << "## a given option"<< std::endl
	   << "a = foo # (custom value)" << std::endl
	   << std::endl
	   << "## a last option" << std::endl
	   << "c = baz # (custom value)" << std::endl
	   << std::endl
	   << std::endl
	   << std::endl
	   << "## but not this one" << std::endl
	   << "[foo.bar]" << std::endl
	   << std::endl
	   << std::endl
	   << std::endl
	   << "[foo.baz]" << std::endl
	   << std::endl
	   << "dd = dd # (custom value)" << std::endl
	   << std::endl;


	std::ostringstream output;
	options::ConfigFileParser p;
	p.Print(output,d_root);
	EXPECT_EQ(os.str(),output.str());

}
