/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * PluginLoader.h
 *
 *  Created on: 6 nov. 2012
 *      Author: akay
 */

#ifndef BIOROB_CPP_PLUGIN_PLUGINLOADER_H_
#define BIOROB_CPP_PLUGIN_PLUGINLOADER_H_

#include <biorob-cpp/base/common.h>

#include <vector>

#include <tr1/memory>

#define BIOROB_CPP_ENTRY_FUNCTION_NAME(PluginName) biorob_cpp_plugin_entry_ ## PluginName
#define BIOROB_CPP_EXIT_FUNCTION_NAME(PluginName) biorob_cpp_plugin_exit_ ## PluginName


#define BIOROB_CPP_DECLARE_PLUGIN(PluginName) \
public : \
	inline static std::string Name() { \
		return #PluginName ; \
	} \
private :

#define BIOROB_CPP_EXPORT_PLUGIN(PluginName,ClassName) \
C_EXPORT PluginName * BIOROB_CPP_ENTRY_FUNCTION_NAME(PluginName) ();\
C_EXPORT void BIOROB_CPP_EXIT_FUNCTION_NAME(PluginName) ( PluginName * p );\
C_EXPORT PluginName * BIOROB_CPP_ENTRY_FUNCTION_NAME(PluginName) (){ \
	return new ClassName(); \
} \
C_EXPORT void BIOROB_CPP_EXIT_FUNCTION_NAME(PluginName) ( PluginName * p ) { \
	delete p; \
}






template <class Plugin>
class PluginLoader {
public:
	typedef std::tr1::shared_ptr<Plugin> PluginPtr;
	enum LoadingHint {
		Default = 0,
		ResolveAllSymbols   = (1 << 0),
		IsCMakeModuleObject = (1 << 1)
	};
	PluginLoader(LoadingHint h = Default);
	virtual ~PluginLoader();

	void SetHint(LoadingHint m);
	void SetFilename(const std::string & name);
	void AppendVariant(const std::string & name);

	PluginPtr LoadPlugin();
private :
	typedef void * DlHandle;
	typedef Plugin * (*EntryFptr)(void);
	typedef void (*ExitFptr)(Plugin*);

	class Deleter {
	public :
		Deleter(DlHandle h,ExitFptr);
		void operator()(Plugin * p);
	private :
		DlHandle d_handle;
		ExitFptr d_exit;
	};


	typedef std::vector<std::string> ListOfVariants;

	DlHandle OpenSharedObject(const std::string & name,
	                          std::ostream & error);
	void * OpenOrReport(const char * o,
	                    int mode,
	                    std::ostream & error);



	LoadingHint    d_hint;
	ListOfVariants d_variants;
	PluginPtr      d_plugin;
};




#include "PluginLoader.ih"

#endif /* BIOROB_CPP_PLUGIN_PLUGINLOADER_H_ */
