/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * Copyright (c) 2012  Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file IsConvertibleTo.h
 * \author Alexandre Tuleu
 * This code is highly inspired of standard C++ technique that can be found on
 * the net. In fact I think it must be in Soustrup's book, Modern C++ Design.
 */
#ifndef BIOROB_CPP_STATIC_IS_CONVERTIBLE_TO_H_
#define BIOROB_CPP_STATIC_IS_CONVERTIBLE_TO_H_

namespace biorobcpp{


	/**
	 * A templated class that could detect at compile time if type U could be
	 * converted to V. U is the class to test.
	 */
	template <typename U,typename V>
	class IsConvertibleTo{
		//I know that coding standard ask for starting to define public interface
		//first. Please look at the public statement
	private :
		typedef char no;
		typedef struct{char dummy[2];} yes;

		static yes test(const V &); // will be chosen if U could be converted to V.
		static no  test(...);
		static const U& helper();

	public :
		/**
		 * value will be statically \true if object of type U can be converted to V
		 */
		static const bool value = sizeof(test(helper())) == sizeof(yes);

	};

}


#endif // BIOROB_CPP_STATIC_IS_CONVERTIBLE_TO_H_
