/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file IsConvertibleUTest.cpp
 *
 *  \date Aug 11, 2011
 *  \author Alexandre Tuleu
 */
#include "IsConvertibleTo.h"
#include "IsConvertibleUTest.h"

#include <biorob-cpp/base/common.h>

TEST_F(IsConvertibleUTest,PrimitiveTypeCorrectlyTested){

	EXPECT_TRUE( (biorobcpp::IsConvertibleTo<unsigned int,int>::value) );
	EXPECT_TRUE( (biorobcpp::IsConvertibleTo<int,unsigned int>::value) );
	EXPECT_TRUE( (biorobcpp::IsConvertibleTo<float,double>::value) );
	EXPECT_TRUE( (biorobcpp::IsConvertibleTo<double,float>::value) );
	EXPECT_TRUE( (biorobcpp::IsConvertibleTo<const char *,std::string>::value) );
	//you cannot convert a C++ string to a C string natively, you use
	//std::string::c_str() for that !
	EXPECT_FALSE( (biorobcpp::IsConvertibleTo<std::string,const char *>::value) );


}



class A {};
class B : public A{};


class C {};
class D : public C{
public :
	//A C could now be casted to a D, D nows how to do it.
	D(const C &){};
};


TEST_F(IsConvertibleUTest,CustomTypeCorrectlyTested){
	//B is convertible to A, if we slice it (B is a A)
	EXPECT_TRUE((biorobcpp::IsConvertibleTo<B,A>::value));
	//A is not convertible to B
	EXPECT_FALSE((biorobcpp::IsConvertibleTo<A,B>::value));

	EXPECT_TRUE((biorobcpp::IsConvertibleTo<D,C>::value));
	EXPECT_TRUE((biorobcpp::IsConvertibleTo<C,D>::value));

}
